"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _bluebirdLst;

function _load_bluebirdLst() {
    return _bluebirdLst = require("bluebird-lst");
}

var _awsSdk;

function _load_awsSdk() {
    return _awsSdk = require("aws-sdk");
}

var _electronBuilderUtil;

function _load_electronBuilderUtil() {
    return _electronBuilderUtil = require("electron-builder-util");
}

var _electronPublish;

function _load_electronPublish() {
    return _electronPublish = require("electron-publish");
}

var _progress;

function _load_progress() {
    return _progress = require("electron-publish/out/progress");
}

var _fsExtraP;

function _load_fsExtraP() {
    return _fsExtraP = require("fs-extra-p");
}

var _path = require("path");

var _uploader;

function _load_uploader() {
    return _uploader = require("./uploader");
}

class S3Publisher extends (_electronPublish || _load_electronPublish()).Publisher {
    constructor(context, info) {
        super(context);
        this.info = info;
        this.providerName = "S3";
        (0, (_electronBuilderUtil || _load_electronBuilderUtil()).debug)(`Creating S3 Publisher — bucket: ${info.bucket}`);
    }
    static checkAndResolveOptions(options, channelFromAppVersion) {
        return (0, (_bluebirdLst || _load_bluebirdLst()).coroutine)(function* () {
            const bucket = options.bucket;
            if (bucket == null) {
                throw new Error(`Please specify "bucket" for "s3" update server`);
            }
            if (bucket.indexOf(".") !== -1 && options.region == null) {
                // on dotted bucket names, we need to use a path-based endpoint URL. Path-based endpoint URLs need to include the region.  
                const s3 = new (_awsSdk || _load_awsSdk()).S3({ signatureVersion: "v4" });
                options.region = (yield s3.getBucketLocation({ Bucket: bucket }).promise()).LocationConstraint;
            }
            if (options.channel == null && channelFromAppVersion != null) {
                options.channel = channelFromAppVersion;
            }
        })();
    }
    // http://docs.aws.amazon.com/sdk-for-javascript/v2/developer-guide/s3-example-creating-buckets.html
    upload(file, safeArtifactName) {
        var _this = this;

        return (0, (_bluebirdLst || _load_bluebirdLst()).coroutine)(function* () {
            const fileName = (0, _path.basename)(file);
            const fileStat = yield (0, (_fsExtraP || _load_fsExtraP()).stat)(file);
            const client = new (_uploader || _load_uploader()).S3Client({ s3Options: { signatureVersion: "v4" } });
            const cancellationToken = _this.context.cancellationToken;
            const uploader = client.createFileUploader(file, (_this.info.path == null ? "" : `${_this.info.path}/`) + fileName, {
                Bucket: _this.info.bucket,
                ACL: _this.info.acl || "public-read",
                StorageClass: _this.info.storageClass || undefined
            });
            const progressBar = _this.createProgressBar(fileName, fileStat);
            if (progressBar != null) {
                const callback = new (_progress || _load_progress()).ProgressCallback(progressBar);
                uploader.on("progress", function () {
                    if (!cancellationToken.cancelled) {
                        callback.update(uploader.loaded, uploader.contentLength);
                    }
                });
            }
            return cancellationToken.createPromise(function (resolve, reject, onCancel) {
                onCancel(function () {
                    return uploader.abort();
                });
                uploader.upload().then(function () {
                    try {
                        (0, (_electronBuilderUtil || _load_electronBuilderUtil()).debug)(`S3 Publisher: ${fileName} was uploaded to ${_this.info.bucket}`);
                    } finally {
                        resolve();
                    }
                }).catch(reject);
            });
        })();
    }
    toString() {
        return `S3 (bucket: ${this.info.bucket})`;
    }
}
exports.default = S3Publisher; //# sourceMappingURL=s3Publisher.js.map